@@ -52,6 +52,8 @@ class EigrpIPField(StrField, IPField):
     EIGRP removes zeros from the host portion of the ip address if the netmask is 8, 16 or 24 bits.
     """
 
+    __slots__ = ["length_from"]
+
     def __init__(self, name, default, length=None, length_from=None):
         StrField.__init__(self, name, default)
         self.length_from  = length_from
@@ -101,25 +103,25 @@ class EigrpIPField(StrField, IPField):
     def i2len(self, pkt, x):
         l = self.length_from(pkt)
         l = self.prefixlen_to_bytelen(l)
-
         return l
 
     def getfield(self, pkt, s):
         l = self.length_from(pkt)
         l = self.prefixlen_to_bytelen(l)
-
         return s[l:], self.m2i(pkt, s[:l])
 
     def randval(self):
         return IPField.randval(self)
 
-class EigrpIP6Field(StrField, IP6Field, EigrpIPField):
+class EigrpIP6Field(StrField, IP6Field):
     """
     This is a special field type for handling ip addresses of destination networks in internal and
     external route updates.
 
     """
 
+    __slots__ = ["length_from"]
+
     def __init__(self, name, default, length=None, length_from=None):
         StrField.__init__(self, name, default)
         self.length_from  = length_from
@@ -163,11 +165,17 @@ class EigrpIP6Field(StrField, IP6Field, EigrpIPField):
         return l
 
     def i2len(self, pkt, x):
-        return EigrpIPField.i2len(self, pkt, x)
+        l = self.length_from(pkt)
+        l = self.prefixlen_to_bytelen(l)
+        return l
 
     def getfield(self, pkt, s):
-        return EigrpIPField.getfield(self, pkt, s)
+        l = self.length_from(pkt)
+        l = self.prefixlen_to_bytelen(l)
+        return s[l:], self.m2i(pkt, s[:l])
 
+    def randval(self):
+        return IP6Field.randval(self)
 
 class EIGRPGeneric(Packet):
     name = "EIGRP Generic TLV"
@@ -259,7 +267,7 @@ class ShortVersionField(ShortField):
 
             return (major << 8) | minor
 
-        elif type(x) is int and x >= 0 and x <= 65535:
+        elif type(x) is int and 0 <= x <= 65535:
             return x
         else:
             if self.default != None:
